@@ -564,7 +564,9 @@ class WorkerAutomation(
                 }
             }
         }
-
+        // A better tile than this unit can build might have been stored in the cache
+        if (!rank.repairImprovment!! && (rank.bestImprovement == null || 
+                !unit.canBuildImprovement(rank.bestImprovement!!, tile))) return -100f
         return rank.improvementPriority!!
     }
 
